
from PyQt4.QtCore import *
from PyQt4.QtGui import * 
import sys
import psycopg
from Ui_newpostgislayer import Ui_NewPostGISLayer
from newpostgislayer_field import NewField

class NewPostGISLayer (QDialog, Ui_NewPostGISLayer):
    MSG_BOX_TITLE = "New PostGIS Layer"

    def __init__(self):
        QDialog.__init__(self)
        self.setupUi(self)
        self.dbport.setText('5432')
        self.dbhost.setText('localhost')
        self.progressBar.setValue(0);
        self.sridlist = {}
        self.columnlist = {}
        wip = QTableWidgetItem('')
        #wip.setFlags(2)
        self.columns.setItemPrototype(wip) 
 
    def on_buttonConnect_pressed(self):
        try:
            self.connection = psycopg.connect(str(self.getDsn()))
        except Exception, e:
            QMessageBox.warning(self, "Error", "Cannot connect to database (" + self.getDsn()  + ")" + " the connector reported :\n" + str(e) ,  QMessageBox.Ok)
        else:
            self.layerDetails()


    def on_buttonCancel_pressed(self):
        #print "CANCEL button in buttonbox"
        self.done(0)
    
    def on_dbhost_textChanged(self):
        self.checkFilled()

    def on_dbname_textChanged(self):
        self.checkFilled()

    def on_dbuser_textChanged(self):
        self.checkFilled()
    
    def on_dbpassword_textChanged(self):
        self.checkFilled()
    
    def on_dbport_textChanged(self):
        self.checkFilled()
    
    def checkFilled(self):
        if (self.dbname.text() \
        and self.dbuser.text() \
        and self.dbpassword.text() \
        and self.dbport.text() \
        and self.dbhost.text()  ):
         self.buttonConnect.setEnabled ( True)

    
    def getDsn(self):
        return 'user=' + self.dbuser.text() + ' password=' + self.dbpassword.text()  + ' port=' + self.dbport.text()  + ' host=' + self.dbhost.text() + ' dbname=' + self.dbname.text();
    
    ## Layer creation methods
    def layerDetails(self):
        """ Get the SRID list """
        try:
            curs = self.connection.cursor()
            curs.execute( 'select count(srid)from spatial_ref_sys')
            recnum = int(curs.fetchone()[0])
            curs.execute( 'select srid, srtext from spatial_ref_sys')
            counter = 0;
            self.progressBar.setMinimum(0)
            self.progressBar.setMaximum(recnum);    
            self.sridlist['-1'] = 'None';
            for p in curs.fetchall(): 
                counter += 1
                self.sridlist[str(p[0])] = "<br />".join(str(p[1]).split(','))
                self.srid.addItem(str(p[0]))                
                self.progressBar.setValue(counter)
            self.on_srid_currentIndexChanged()
        except Exception, e:
            QMessageBox.warning(self, "Error", "Cannot fetch SRID list from the database,\nthe connector reported :\n" + str(e) + "\n" + "Are you sure the database you specified is spatially enabled?" ,  QMessageBox.Ok)
        
        self.layerbox.setEnabled (True)
        self.srid.setEnabled (True)
        self.layername.setEnabled (True)
        self.layertype.setEnabled (True)
        self.pkcol.setEnabled (True)
        
    
    def on_srid_currentIndexChanged(self):
        if len(self.sridlist) :
            self.sriddesc.setText(self.sridlist[str(self.srid.currentText())])

    def on_layertype_currentIndexChanged(self):
        self.checkFilledLayer()

    def on_pkcol_currentIndexChanged(self):
        self.checkFilledLayer()

    def on_layername_textChanged(self):
        self.checkFilledLayer()

    def on_geomcol_textChanged(self):
        self.checkFilledLayer()

    def checkFilledLayer(self):
        if (self.layername.text()  <> '' and self.geomcol.text() <> ''):
            self.updateColumnTable()
            self.buttonOk.setEnabled(True)
        else:
           self.buttonOk.setEnabled(False) 
    
    def on_buttonAddcolumn_pressed(self):
        """ Add a table column """
        ui = NewField()
        #ui.setupUi(self)
        ui.show()

    def on_buttonRemovecolumn_pressed(self):
        """ Remove a table column, check for PK and geometry col """
        pkcol       = str(self.pkcol.currentText())
        geomcol     = str(self.geomcol.text())
        item = self.columns.currentItem()
        print "R %s - C %s" % (item.row(), item.column())
        if item.row() <= 1:
            QMessageBox.warning(self, "Error", "Cannot delete primary key id or geometry column fields" ,  QMessageBox.Ok)
        else:
            item_name = self.columns.item(item.row(), 0).name
            self.columns.removeRow(item.row())
            del(self.columnlist[str(self.columns.item(0,0).text())])
    

    def updateColumnTable(self):
        srid        = str(self.sridlist[str(self.srid.currentText())])
        pkcol       = str(self.pkcol.currentText())
        layertype   = str(self.layertype.currentText())
        geomcol     = str(self.geomcol.text())
        self.columnlist['pkcol'] = {'name' : pkcol, 'type' : 'int', 'size' : '4', 'null' : False, 'editable' : False} 
        self.columnlist['geomcol'] = {'name' : geomcol, 'type' : layertype , 'size' : '', 'null' : False, 'editable' : False}
        rowcounter = 0
        
        self.columns.setSortingEnabled(False)
        for columndata in self.columnlist.values():
            if self.columns.rowCount() <= rowcounter:
                self.columns.insertRow(rowcounter)
            self.columns.setItem(rowcounter, 0, QTableWidgetItem(columndata['name']))
            self.columns.setItem(rowcounter, 1, QTableWidgetItem(columndata['type']))
            self.columns.setItem(rowcounter, 2, QTableWidgetItem(columndata['size']))
            self.columns.setItem(rowcounter, 3, (QTableWidgetItem('True' if columndata['null'] else 'False')))
                #self.columns.setItem(rowcounter, 4, QTableWidgetItem(columndata['name']))
            rowcounter += 1
        
        self.columns.setSortingEnabled(True)
   
    def on_buttonOk_pressed(self):
        """ Create the new layer """
        layername  = str(self.layername.text())
        srid       = str(self.srid.currentText())
        pkcol      = str(self.pkcol.currentText())
        layertype  = str(self.layertype.currentText())
        geomcol    = str(self.geomcol.text())
        try:
            curs = self.connection.cursor()        
            curs.execute( 'CREATE TABLE ' + layername + ' (' + pkcol + ' int4 primary key)')
            curs.execute("select AddGeometryColumn('" + layername + \
            "', '" + geomcol + "', " + srid + ", '" + layertype + "', 2)")
            
            # create the GIST index
            curs.execute("create index sidx_" + layername + " on " + \
            layername + " USING GIST(the_geom GIST_GEOMETRY_OPS)")
            self.connection.commit()
        except Exception, e:
            QMessageBox.warning(self, "Error", "Cannot cannot create the new PostGIS table,\nthe connector reported :\n" + str(e) ,  QMessageBox.Ok)
        else:
            QMessageBox.warning(self, "Success", "Layer " + layername +  " has been successfully created" ,  QMessageBox.Ok)
            # Exit
            self.done(0)

## Class end
    
if __name__ == "__main__":
    app = QApplication(sys.argv)
    ui = NewPostGISLayer()
    
    ui.show()
    sys.exit(app.exec_())
